from __future__ import print_function, division

import os
try:
    import cPickle as pickle
except ImportError:
    import pickle

import numpy as np
from astropy.table import Table

from .fit_info import FitInfo
from .extinction import Extinction
from .models import load_parameter_table


def write_parameters(input_file, output_file, select_format=("N", 1), additional={}):
    """
    Write out an ASCII file with the paramters for each source.

    Parameters
    ----------
    input_file : str
        File containing the fit information
    output_file : str, optional
        The output ASCII file containing the parameters
    select_format : tuple, optional
        Tuple specifying which fits should be output. See the documentation
        for a description of the tuple syntax.
    additional : dict, optional
        A dictionary giving additional parameters for each model. This should
        be a dictionary where each key is a parameter, and each value is a
        dictionary mapping the model names to the parameter values.
    """

    # Open input and output file
    fin = open(input_file, 'rb')
    fout = open(output_file, 'w')

    # Read in header of output file
    model_dir = pickle.load(fin)
    filters = pickle.load(fin)
    extinction = pickle.load(fin)

    # Read in table of parameters for model grid
    t = load_parameter_table(model_dir)

    t['MODEL_NAME'] = np.char.strip(t['MODEL_NAME'])
    t.sort('MODEL_NAME')

    # First header line
    fout.write("source_name".center(30) + ' ')
    fout.write("n_data".center(10) + ' ')
    fout.write("n_fits".center(10) + ' ')

    fout.write('\n')

    # Second header line

    fout.write('fit_id'.center(10) + ' ')
    fout.write('model_name'.center(30) + ' ')
    fout.write('chi2'.center(10) + ' ')
    fout.write('av'.center(10) + ' ')
    fout.write('scale'.center(10) + ' ')

    for par in list(t.columns.keys()) + list(additional.keys()):
        if par == 'MODEL_NAME':
            continue
        fout.write(par.lower().center(10) + ' ')

    fout.write('\n')

    fout.write('-' * (75 + 11 * (len(list(t.columns.keys()) + list(additional.keys())))))
    fout.write('\n')

    while True:

        # Read in next fit
        try:
            info = pickle.load(fin)
        except EOFError:
            break

        # Filter fits
        info.keep(select_format[0], select_format[1])

        # Get filtered and sorted table of parameters
        tsorted = info.filter_table(t, additional=additional)

        fout.write("%30s " % info.source.name)
        fout.write("%10i " % info.source.n_data)
        fout.write("%10i " % info.n_fits)
        fout.write("\n")

        for fit_id in range(len(info.chi2)):
            fout.write('%10i ' % (fit_id + 1))
            fout.write('%30s ' % info.model_name[fit_id])
            fout.write('%10.3f ' % info.chi2[fit_id])
            fout.write('%10.3f ' % info.av[fit_id])
            fout.write('%10.3f ' % info.sc[fit_id])

            for par in tsorted.columns:
                if par == 'MODEL_NAME':
                    continue
                fout.write('%10.3e ' % (tsorted[par][fit_id]))

            fout.write('\n')

    # Close input and output files
    fin.close()
    fout.close()
    fout.close()



from __future__ import print_function, division

try:
    import cPickle as pickle
except ImportError:
    import pickle

from .fit_info import FitInfo
from .extinction import Extinction


def filter_output(input_file=None, output_good='auto', output_bad='auto', chi=None,
                  cpd=None):
    """
    Filter an output file into well and badly fit sources.

    The sources in the output file are split into ones where the best fit is
    good, and those where the best fit is bad, where the distinction between
    good and bad is made based on the chi^2 value or the chi^2 value per
    datapoint of the best fit.

    Parameters
    ----------
    input_file : str
        File containing the fit information
    output_good : str, optional
        The name of the file containing information about well-fit sources. If
        set to 'auto', then the output file will be set to the same as the
        input file but with a ``_good`` prefix.
    output_bad : str, optional
        The name of the file containing information about badly-fit sources. If
        set to 'auto', then the output file will be set to the same as the
        input file but with a ``_bad`` prefix.
    chi : float, optional
        If set, the separation between well and badly fit sources is done based
        on the total chi^2 of the best fit, using this value as a threshold.
    cpd : float, optional
        If set, the separation between well and badly fit sources is done based
        on the chi^2 per datapoint of the best fit, using this value as a
        threshold.
    """

    fin = open(input_file, 'rb')

    if output_good == 'auto':
        fout_good = open(input_file + '_good', 'wb')
    else:
        fout_good = open(output_good, 'wb')

    if output_bad == 'auto':
        fout_bad = open(input_file + '_bad', 'wb')
    else:
        fout_bad = open(output_bad, 'wb')

    # Read in header of output file
    model_dir = pickle.load(fin)
    filters = pickle.load(fin)
    extinction_law = pickle.load(fin)

    # Output header to good fits file
    pickle.dump(model_dir, fout_good, 2)
    pickle.dump(filters, fout_good, 2)
    pickle.dump(extinction_law, fout_good, 2)

    # Output header to bad fits file
    pickle.dump(model_dir, fout_bad, 2)
    pickle.dump(filters, fout_bad, 2)
    pickle.dump(extinction_law, fout_bad, 2)

    while True:

        # Read in next fit
        try:
            info = pickle.load(fin)
        except EOFError:
            break

        bestchi = info.chi2[0]
        bestcpd = info.chi2[0] / float(info.source.n_data)

        if (chi and bestchi < chi) or (cpd and bestcpd < cpd):
            pickle.dump(info, fout_good, 2)
        else:
            pickle.dump(info, fout_bad, 2)

    # Close input and output files
    fin.close()
    fout_good.close()
    fout_bad.close()
